
#ifndef PARTICLESYSTEM_H
#define PARTICLESYSTEM_H

//// Includes - System
#include <GL/gl.h>

//// Includes - Project
#include "vector3f.h"

struct Particle
{
    vector3f pos;
    vector3f vel;
	float	age;

	Particle *pNext;
};

class SimpleLinkedList
{
public:
	SimpleLinkedList();
	~SimpleLinkedList();

	void push(SimpleLinkedList *node);
	SimpleLinkedList* nextNode();

private:
	SimpleLinkedList* next;
};

//// Classes ////
class ParticleSystem
{
public:
	// Set-methods for our system

	ParticleSystem();
	~ParticleSystem();

	void setGravity( vector3f in_grav );
	void setWind( vector3f in_wind );
	void setWindState( bool power);
	void setInitialVelocity ( vector3f in_initVel );
	void setInitialPosition ( vector3f in_initPos );
	void setParticleSize( float size);
	void setRandomFactor( float perc);
	void setRandomOn( bool power);
	void setBoxCorner ( vector3f in_initPos  );
	void setReleaseInterval ( float in_lim );
	void setReleaseNumber ( int in_num );
	void setParticleCount ( int in_partnum );
	void setParticleLife ( float in_partlife );
	void setFade( float perc);
	void setTexture ( char *in_texturename );
	
	void init();
	void update(double in_timePassed );
	void render();

	//void reset();

private:
	Particle*	moveToFreeList(Particle *part);
	float		rndFloatBetween( float min, float max );
	inline void initParticle(Particle *in_part);
	bool		checkExtentions();

	// OpenGL Related variables
	GLuint		pTexture;
	float		maxSize;

	// Particle System Properties
	Particle	*partFreeList;
	Particle	*partActiveList;

	int			numParticles;
	int			activeParticles;

	float		pSysAge;
	float       pSysLastItAge;

	float		pRelInterval;
	int			pRelNum;

	// Particle System - particle properties
	float		pTTL;

	vector3f	pInitPos;
	vector3f	pBoxCorner;
	bool		boxDistribution;
	vector3f	pInitVel;
	vector3f	pGrav;

	vector3f	pWind;
	bool		pWindOn;

	float		pSysRandomFactor;
	bool		pSysRndOn;
	float		pSize;

	float		fadeFact;
};
#endif